@@ -4,7 +4,6 @@ from __future__ import unicode_literals
 import functools
 import inspect
 import logging
-from collections import namedtuple
 from operator import attrgetter
 
 import flask
@@ -12,6 +11,7 @@ from decimal import Decimal
 from flask_wtf import FlaskForm as BaseForm
 from keg.db import db
 import sqlalchemy as sa
+from markupsafe import Markup
 from sqlalchemy_utils import ArrowType
 import six
 import wtforms.fields
@@ -20,6 +20,7 @@ from wtforms.validators import InputRequired, Optional, StopValidation, NumberRa
 from wtforms_alchemy import model_form_factory, FormGenerator as FormGeneratorBase
 from wtforms_components.fields import SelectField as SelectFieldBase
 
+from keg_elements.db.columns import DBEnum
 from keg_elements.extensions import lazy_gettext as _
 from keg_elements.forms.validators import NumberScale
 
@@ -118,8 +119,7 @@ class FieldMeta(object):
         elif modifier is None:
             choices = default_choices
         else:
-            map_func = lambda pair: (pair[0], modifier(pair[1]))
-            choices = map(map_func, default_choices)
+            choices = [(v, modifier(l)) for v, l in default_choices]
 
         field.kwargs['choices'] = self.modify_choices(choices)
 
@@ -274,11 +274,11 @@ class TypeHintingTextInputB3(_TypeHintingTextInputBase):
     """
     def __call__(self, field, **kwargs):
         def make_addon(txt):
-            return wtforms.widgets.HTMLString(
+            return Markup(
                 '<span class="input-group-addon">{}</span>'.format(wtforms.widgets.core.escape(txt))
             )
 
-        return wtforms.widgets.HTMLString(
+        return Markup(
             '<div class="input-group">{pre}{field}{post}</div>'.format(
                 pre=make_addon(self.prefix) if self.prefix else '',
                 field=super().__call__(field, **kwargs).__html__(),
@@ -294,13 +294,13 @@ class TypeHintingTextInputB4(_TypeHintingTextInputBase):
     """
     def __call__(self, field, **kwargs):
         def make_addon(txt, addon_type):
-            return wtforms.widgets.HTMLString(
+            return Markup(
                 '<div class="input-group-{type}">'
                 '   <span class="input-group-text">{txt}</span>'
                 "</div>".format(type=addon_type, txt=wtforms.widgets.core.escape(txt))
             )
 
-        return wtforms.widgets.HTMLString(
+        return Markup(
             '<div class="input-group">{pre}{field}{post}</div>'.format(
                 pre=make_addon(self.prefix, "prepend") if self.prefix else "",
                 field=super().__call__(field, **kwargs).__html__(),
@@ -365,6 +365,8 @@ class FormGenerator(FormGeneratorBase):
     def create_validators(self, prop, column):
         validators = super(FormGenerator, self).create_validators(prop, column)
         if isinstance(column.type, sa.Numeric) and not isinstance(column.type, sa.Float):
+            if column.type.precision is None or column.type.scale is None:
+                raise ValueError('Numeric fields must specify precision and scale')
             max_ = _max_for_numeric(column.type.precision, column.type.scale)
             validators.append(NumberRange(min=-max_, max=max_))
             validators.append(NumberScale(column.type.scale))
@@ -375,6 +377,15 @@ class FormGenerator(FormGeneratorBase):
             return None
         return super(FormGenerator, self).length_validator(column)
 
+    def select_field_kwargs(self, column):
+        enum_cls = getattr(column.type, 'enum_class', None)
+        if enum_cls and issubclass(enum_cls, DBEnum):
+            return {
+                'coerce': enum_cls.coerce,
+                'choices': enum_cls.form_options()
+            }
+        return super().select_field_kwargs(column)
+
 
 def field_to_dict(field):
     if isinstance(field, wtforms.fields.FormField):
@@ -404,7 +415,7 @@ def form_validator(func=None, only_when_fields_valid=False):
 
     @functools.wraps(func)
     def wrapper(form):
-        if not only_when_fields_valid or not form.errors:
+        if not only_when_fields_valid or not form.field_errors:
             return func(form)
 
     global ___validator_creation_counter
@@ -499,14 +510,15 @@ class Form(BaseForm):
         return self._form_level_errors
 
     @property
-    def errors(self):
-        if self._errors is None:
-            self._errors = {name: f.errors for name, f in six.iteritems(self._fields) if f.errors}
-        return self._errors
+    def field_errors(self):
+        return super().errors
 
     @property
-    def all_errors(self):
-        return namedtuple('Errors', ['field', 'form'])(self.errors, self.form_errors)
+    def errors(self):
+        errors = self.field_errors
+        if self.form_errors:
+            errors['_form'] = self.form_errors
+        return errors
 
 
 BaseModelForm = model_form_factory(Form, form_generator=FormGenerator)
