@@ -5,7 +5,6 @@ import six
 import sqlalchemy as sa
 from sqlalchemy_utils import ArrowType
 
-from keg_elements.sentry import SentryClient
 from .db.utils import validate_unique_exc
 from .db.mixins import DefaultColsMixin
 
@@ -130,6 +129,15 @@ class EntityBase(object):
                 assert col.onupdate, 'Column "{}" should have onupdate set'.format(col.name)
             assert col.server_default, 'Column "{}" should have server_default set'.format(col.name)
 
+    def test_column_numeric_scale_precision_set(self):
+        for col_check in self.column_check_generator():
+            col = getattr(self.entity_cls, col_check.name)
+            if isinstance(col.type, sa.Numeric) and not isinstance(col.type, sa.Float):
+                assert col.type.precision is not None, \
+                    'Column "{}" does not specify precision'.format(col.name)
+                assert col.type.scale is not None, \
+                    'Column "{}" does not specify scale'.format(col.name)
+
     def test_all_columns_are_constraint_tested(self):
         """Checks that all fields declared on entity are in the constraint tests"""
 
@@ -173,17 +181,3 @@ class EntityBase(object):
         except Exception as e:
             if not validate_unique_exc(e):
                 raise
-
-
-class SentryCapture:
-    def __enter__(self):
-        SentryClient.__log_reports__ = True
-        SentryClient.__report_log__ = []
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        SentryClient.__log_reports__ = False
-
-    @property
-    def reports(self):
-        return SentryClient.__report_log__
