@@ -19,6 +19,7 @@ class TestDefaultColsMixin:
     def setup_method(self, fn):
         ents.Thing.delete_cascaded()
 
+    @pytest.mark.skipif(db.engine.dialect.name != 'sqlite', reason='SQLite only test')
     def test_default_ordering(self):
         thing1 = ents.Thing.testing_create(id=6)
         thing2 = ents.Thing.testing_create(id=5)
@@ -174,13 +175,13 @@ class TestMethodsMixin:
         obj = ents.Thing.testing_create()
 
         expected = {'id', 'name', 'color', 'scale_check', 'float_check', 'updated_utc',
-                    'created_utc', 'units'}
+                    'created_utc', 'units', 'status'}
         assert set(obj.to_dict().keys()) == expected
         assert set(obj.to_dict(exclude={'id'}).keys()) == expected - {'id'}
 
         hybrids = ['name_and_color']
         exclude = ['name', 'scale_check', 'float_check', 'color', 'updated_utc', 'created_utc',
-                   'units']
+                   'units', 'status']
         dictionary = obj.to_dict(hybrids=hybrids, exclude=exclude)
 
         assert dictionary == {
