@@ -1,18 +1,18 @@
 import codecs
 import enum
 import random
+import uuid
 
 import sqlalchemy as sa
-
 from keg.db import db
-import keg_elements.db.mixins as mixins
-import keg_elements.db.columns as columns
 
+import keg_elements.db.columns as columns
+import keg_elements.db.mixins as mixins
 
 many_things_mapper = db.Table(
     'many_things_mapper',
-    sa.Column('thing_id', sa.Integer, sa.ForeignKey('things.id')),
-    sa.Column('many_id', sa.Integer, sa.ForeignKey('many_things.id'))
+    sa.Column('thing_id', sa.Integer, sa.ForeignKey('things.id', ondelete='cascade')),
+    sa.Column('many_id', sa.Integer, sa.ForeignKey('many_things.id', ondelete='cascade'))
 )
 
 
@@ -22,6 +22,15 @@ class Units(enum.Enum):
     meters = 'm'
 
 
+class ThingStatus(columns.DBEnum):
+    open = 'Open'
+    closed = 'Closed'
+
+    @classmethod
+    def db_name(cls):
+        return 'enum_thing_status'
+
+
 def random_scale_check():
     return 12.3456
 
@@ -34,6 +43,7 @@ class Thing(db.Model, mixins.DefaultMixin):
     scale_check = db.Column(db.Numeric(8, 4), info=dict(randomdata=random_scale_check))
     float_check = db.Column(db.Float)
     units = db.Column(sa.Enum(Units, name='enum_units'))
+    status = db.Column(ThingStatus.db_type())
 
     @sa.ext.hybrid.hybrid_property
     def name_and_color(self):
@@ -47,13 +57,20 @@ class Thing(db.Model, mixins.DefaultMixin):
     def random_color(cls):
         return 'blue'
 
+    @classmethod
+    def delete_cascaded(cls):
+        RelatedThing.delete_cascaded()
+        AncillaryA.delete_cascaded()
+        AncillaryB.delete_cascaded()
+        super().delete_cascaded()
+
 
 class RelatedThing(db.Model, mixins.DefaultMixin):
     __tablename__ = 'related_things'
 
     name = db.Column(db.Unicode(50), nullable=False)
     is_enabled = db.Column(db.Boolean, nullable=False, default=False,
-                           server_default=sa.text('FALSE'))
+                           server_default=sa.false())
 
     thing_id = sa.Column(sa.Integer, sa.ForeignKey(Thing.id), nullable=False)
     thing = sa.orm.relationship(lambda: Thing, backref=sa.orm.backref(
@@ -67,6 +84,27 @@ class RelatedThing(db.Model, mixins.DefaultMixin):
         return super(RelatedThing, cls).testing_create(**kwargs)
 
 
+class OtherThing(db.Model, mixins.DefaultMixin):
+    __tablename__ = 'other_things'
+
+    name = db.Column(db.Unicode(50), nullable=False)
+    is_enabled = db.Column(db.Boolean, nullable=False, default=False,
+                           server_default=sa.false())
+    unique_field = db.Column(db.Unicode(50), unique=True, nullable=False)
+
+    thing_id = sa.Column(sa.Integer, sa.ForeignKey(Thing.id, ondelete='cascade'), nullable=False)
+    thing = sa.orm.relationship(lambda: Thing, backref=sa.orm.backref(
+        'other_things',
+        cascade='all, delete-orphan'
+    ))
+
+    @classmethod
+    def testing_create(cls, **kwargs):
+        kwargs['thing'] = kwargs.get('thing') or Thing.testing_create()
+        kwargs.setdefault('unique_field', str(uuid.uuid4()))
+        return super(OtherThing, cls).testing_create(**kwargs)
+
+
 class ManyToManyThing(db.Model, mixins.DefaultMixin):
     __tablename__ = 'many_things'
 
@@ -131,6 +169,11 @@ class AncillaryA(mixins.DefaultMixin, db.Model):
 
         return super(AncillaryA, cls).testing_create(**kwargs)
 
+    @classmethod
+    def delete_cascaded(cls):
+        UsesBoth.delete_cascaded()
+        super().delete_cascaded()
+
 
 class AncillaryB(mixins.DefaultMixin, db.Model):
     __tablename__ = 'ancillary_bs'
@@ -148,17 +191,24 @@ class AncillaryB(mixins.DefaultMixin, db.Model):
 
         return super(AncillaryB, cls).testing_create(**kwargs)
 
+    @classmethod
+    def delete_cascaded(cls):
+        UsesBoth.delete_cascaded()
+        super().delete_cascaded()
 
+
+
+# Foreign keys to non-unique, non-primary key columns are not supported by MSSQL
 class UsesBoth(mixins.DefaultMixin, db.Model):
     __tablename__ = 'uses_boths'
     __table_args__ = (
         sa.ForeignKeyConstraint(
-            ('thing_id', 'ancillary_a_id'),
-            (AncillaryA.thing_id, AncillaryA.id)
+            ('ancillary_a_id', 'thing_id'),
+            (AncillaryA.id, AncillaryA.thing_id)
         ),
         sa.ForeignKeyConstraint(
-            ('thing_id', 'ancillary_b_id'),
-            (AncillaryB.thing_id, AncillaryB.id)
+            ('ancillary_b_id', 'thing_id'),
+            (AncillaryB.id, AncillaryB.thing_id)
         ),
     )
 
@@ -219,3 +269,9 @@ class SoftDeleteTester(mixins.SoftDeleteMixin, mixins.DefaultMixin, db.Model):
     def testing_create(cls, **kwargs):
         kwargs['hdp_id'] = kwargs.get('hpd_id') or HardDeleteParent.testing_create().id
         return super().testing_create(**kwargs)
+
+
+class DefaultNumeric(mixins.DefaultMixin, db.Model):
+    __tablename__ = 'default_numeric'
+
+    number = sa.Column(sa.Numeric)
